package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.distributed.print.business.bean.dto.*;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Date: 2021/4/20.
 */
@Service
public class ClientServiceImpl implements ClientService {

    @Autowired
    private ExamTaskService examTaskService;

    @Autowired
    private ExamTaskDetailService examTaskDetailService;

    @Autowired
    private ClientStatusService clientStatusService;

    @Autowired
    private ExamDetailService examDetailService;

    @Autowired
    private ExamDetailCourseService examDetailCourseService;

    @Autowired
    private BasicAttachmentService basicAttachmentService;

    @Autowired
    private CommonService commonService;

    @Autowired
    private ExamPrintPlanService examPrintPlanService;

    @Override
    public IPage<ClientExamTaskDto> listTryTask(Long schoolId, String machineCode, Long orgId, Long printPlanId, String courseCode, String paperNumber, Boolean isTry, Boolean isPass, Integer pageNumber, Integer pageSize) {
        return examTaskService.listTryTask(schoolId, machineCode, orgId, printPlanId, courseCode, paperNumber, isTry, isPass, pageNumber, pageSize);
    }

    @Override
    public String getUrl(Long schoolId, Long examTaskId) {
        return examTaskDetailService.getUrl(schoolId, examTaskId);
    }

    @Override
    public Boolean tagPass(Long schoolId, Long examTaskId, String machineCode, Boolean isPass, Long userId) {
        return clientStatusService.tagPass(schoolId, examTaskId, machineCode, isPass, userId);
    }

    @Override
    public Boolean updatePrintProgress(Long schoolId, Long examDetailId, Integer printProgress) {
        return examDetailService.updatePrintProgress(schoolId, examDetailId, printProgress);
    }

    @Override
    public IPage<ClientExamStudentDto> listStudent(Long schoolId, Long examDetailId, String ticketNumber, String studentName, String courseCode, Integer pageNumber, Integer pageSize) {
        return examDetailService.listStudent(schoolId, examDetailId, ticketNumber, studentName, courseCode, pageNumber, pageSize);
    }


    @Override
    public Map<String, Object> getReprintData(Long schoolId, Long examDetailId, String ticketNumber, String type) {
        Map<String, Object> finalMap = new HashMap<>();
        // 取试卷
        Map<String, String> map = mapCourseUrl(schoolId, examDetailId);
        // 取考生
        List<Map<String, String>> studentList = examDetailService.listStudentByExamDetailId(schoolId, examDetailId, ticketNumber, type);
        // 生成试卷List
        List<ClientPrintDataDto> paperList = studentList.stream().map(m -> {
            ClientPrintDataDto printDataDto = new ClientPrintDataDto();
            String courseCode = m.get("courseCode");
            String courseName = m.get("courseName");
            String paperNumber = m.get("paperNumber");
            String studentName = m.get("studentName");
            String studentCode = m.get("studentCode");
            String paperType = m.get("paperType");
            printDataDto.setCourseCode(courseCode);
            printDataDto.setCourseName(courseName);
            printDataDto.setStudentName(studentName);
            printDataDto.setStudentCode(studentCode);
            printDataDto.setPaperType(paperType);

            StringJoiner sj = new StringJoiner(":");
            String key = sj.add(courseCode).add(paperNumber).add(paperType).toString();
            printDataDto.setUrl(map.get(key));
            return printDataDto;
        }).collect(Collectors.toList());
        finalMap.put("paper", paperList);
        // 生成题卡List
        List<ClientPrintDataDto> cardList = studentList.stream().map(m -> {
            ClientPrintDataDto printDataDto = new ClientPrintDataDto();
            String courseCode = m.get("courseCode");
            String courseName = m.get("courseName");
            String studentName = m.get("studentName");
            String studentCode = m.get("studentCode");
            String paperType = m.get("paperType");
            String attachmentId = m.get("attachmentId");
            printDataDto.setCourseCode(courseCode);
            printDataDto.setCourseName(courseName);
            printDataDto.setStudentName(studentName);
            printDataDto.setStudentCode(studentCode);
            printDataDto.setPaperType(paperType);
            Map<String, String> urlMap = commonService.filePreview(attachmentId, false);
            printDataDto.setUrl(urlMap.get("pathUrl"));
            return printDataDto;
        }).collect(Collectors.toList());
        finalMap.put("card", cardList);
        //
        ExamDetail examDetail = examDetailService.getById(examDetailId);
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(examDetail.getPrintPlanId());
        String variableContent = examPrintPlan.getVariableContent();
        String ordinaryContent = examPrintPlan.getOrdinaryContent();
        List<Map> variableList = JSONObject.parseArray(variableContent, Map.class);
        List<Map> otherList = new ArrayList<>();
        for (Map variable : variableList) {
            Map vMap = new HashMap();
            vMap.put("type", variable.get("type"));
            Map<String, String> urlMap = commonService.filePreview(variable.get("attachmentId").toString(), false);
            vMap.put("htmlUrl", urlMap.get("htmlUrl"));
            vMap.put("pdfUrl", urlMap.get("pdfUrl"));
            otherList.add(vMap);
        }
        List<Map> ordinaryList = JSONObject.parseArray(ordinaryContent, Map.class);
        for (Map ordinary : ordinaryList) {
            Map vMap = new HashMap();
            vMap.put("type", ordinary.get("type"));
            Map<String, String> urlMap = commonService.filePreview(ordinary.get("attachmentId").toString(), false);
            vMap.put("htmlUrl", urlMap.get("htmlUrl"));
            vMap.put("pdfUrl", urlMap.get("pdfUrl"));
            otherList.add(vMap);
        }
        finalMap.put("other", otherList);
        return finalMap;
    }

    @Override
    public IPage<ClientPrintTaskDto> listClientPrintTask(Long schoolId, Long machineCode, String orgId, String printPlanId, String status, String courseCode, String paperNumber, String examPlace, String examRoom, Long examStartTime, Long examEndTime, Boolean isDownload, Boolean validate, Integer pageNumber, Integer pageSize) {
        Set<Long> orgIds = commonService.listSubOrgIds(null);
        Page<ClientPrintTaskDto> page = new Page<>(pageNumber, pageSize);
        IPage<ClientPrintTaskDto> pirntTaskDtoIPage = examPrintPlanService.listClientPrintTask(page, schoolId,machineCode, printPlanId, status, courseCode, paperNumber, examPlace, examRoom, examStartTime, examEndTime,isDownload,validate, orgIds);
        return pirntTaskDtoIPage;
    }

    @Override
    public Map<String, Object> getPrintData(Long schoolId, Long examDetailId, String machineCode, Boolean isForceTry, String printUser) {
        // 1.校验是否打样
        List<ExamDetailCourse> examDetailCourses = examDetailCourseService.listByExamDetailId(examDetailId);
        if(isForceTry){
            for (ExamDetailCourse examDetailCours : examDetailCourses) {
                QueryWrapper<ClientStatus> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(ClientStatus::getSchoolId, schoolId)
                        .eq(ClientStatus::getCourseCode, examDetailCours.getCourseCode())
                        .eq(ClientStatus::getPaperNumber, examDetailCours.getPaperNumber())
                        .eq(ClientStatus::getMachineCode, machineCode);
                ClientStatus clientStatuse = clientStatusService.getOne(queryWrapper);
                if(clientStatuse == null){
                    throw ExceptionResultEnum.ERROR.exception("该机器没有打样");
                } else {
                    if (!clientStatuse.getPass()) {
                        throw ExceptionResultEnum.ERROR.exception("该机器打样不合格");
                    }
                }
            }
        }

        Map<String, Object> finalMap = new HashMap<>();
        // 2.取生成的完整的pdf
        ExamDetail examDetail = examDetailService.getById(examDetailId);
        if(examDetail.getAttachmentId() == null) {
            throw ExceptionResultEnum.ERROR.exception("考场pdf未生成");
        }

        BasicAttachment attachment = basicAttachmentService.getById(examDetail.getAttachmentId());
        if(attachment == null){
            throw ExceptionResultEnum.ERROR.exception("考场pdf文件记录异常");
        }

        Map<String, String> attachMap = commonService.filePreview(examDetail.getAttachmentId().toString(), false);
        String totalPathUrl = attachMap.get("pathUrl");
        if(StringUtils.isNotBlank(totalPathUrl)) {
            throw ExceptionResultEnum.ERROR.exception("考场pdf文件丢失");
        }

        finalMap.put("total", totalPathUrl);

        // 3.取学生试卷、题卡、备用试卷、题卡，印品
        Map<String, Object> detailMap = new HashMap<>();
        // 3.1取试卷
        Map<String, String> map = mapCourseUrl(schoolId, examDetailId);
        // 3.2取考生
        List<Map<String, String>> studentList = examDetailService.listStudentByExamDetailId(schoolId, examDetailId, "1", "1");
        // 3.3生成试卷List
        List<ClientPrintDataDto> paperList = studentList.stream().map(m -> {
            ClientPrintDataDto printDataDto = new ClientPrintDataDto();
            String courseCode = m.get("courseCode");
            String courseName = m.get("courseName");
            String paperNumber = m.get("paperNumber");
            String studentName = m.get("studentName");
            String studentCode = m.get("studentCode");
            String paperType = m.get("paperType");
            printDataDto.setCourseCode(courseCode);
            printDataDto.setCourseName(courseName);
            printDataDto.setStudentName(studentName);
            printDataDto.setStudentCode(studentCode);
            printDataDto.setPaperType(paperType);

            StringJoiner sj = new StringJoiner(":");
            String key = sj.add(courseCode).add(paperNumber).add(paperType).toString();
            printDataDto.setUrl(map.get(key));
            return printDataDto;
        }).collect(Collectors.toList());
        detailMap.put("paper", paperList);
        // 3.4生成题卡List
        List<ClientPrintDataDto> cardList = studentList.stream().map(m -> {
            ClientPrintDataDto printDataDto = new ClientPrintDataDto();
            String courseCode = m.get("courseCode");
            String courseName = m.get("courseName");
            String studentName = m.get("studentName");
            String studentCode = m.get("studentCode");
            String paperType = m.get("paperType");
            String attachmentId = m.get("attachmentId");
            printDataDto.setCourseCode(courseCode);
            printDataDto.setCourseName(courseName);
            printDataDto.setStudentName(studentName);
            printDataDto.setStudentCode(studentCode);
            printDataDto.setPaperType(paperType);
            Map<String, String> urlMap = commonService.filePreview(attachmentId, false);
            printDataDto.setUrl(urlMap.get("pathUrl"));
            return printDataDto;
        }).collect(Collectors.toList());
        detailMap.put("card", cardList);

        //3.4备用试卷、题卡
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(examDetail.getPrintPlanId());
        Map<String, Object> backupMap = new HashMap<>();
        int backupCount = examPrintPlan.getBackupCount().intValue();
        //试卷
        List<Map<String, String>> keyMaps = studentList.stream().map(m->{
            Map<String, String> stringMap = new HashMap<>();
            stringMap.put("courseCode", m.get("courseCode"));
            stringMap.put("courseName", m.get("courseName"));
            stringMap.put("paperNumber", m.get("paperNumber"));
            stringMap.put("paperType", m.get("paperType"));
            return stringMap;
        }).distinct().collect(Collectors.toList());
        List<ClientPrintBackupDataDto> paperBackupList = new ArrayList<>();
        for (Map<String, String> keyMap : keyMaps) {
            for (int i = 0; i < backupCount; i++) {
                ClientPrintBackupDataDto clientPrintBackupDataDto = new ClientPrintBackupDataDto();
                clientPrintBackupDataDto.setCourseCode(keyMap.get("courseCode"));
                clientPrintBackupDataDto.setCourseName(keyMap.get("courseName"));
                clientPrintBackupDataDto.setPaperNumber(keyMap.get("paperNumber"));
                clientPrintBackupDataDto.setPaperType(keyMap.get("paperType"));
                StringJoiner sj = new StringJoiner(":");
                String key = sj.add(keyMap.get("courseCode")).add(keyMap.get("paperNumber")).add(keyMap.get("paperType")).toString();
                clientPrintBackupDataDto.setUrl(map.get(key));
                paperBackupList.add(clientPrintBackupDataDto);
            }
        }

        backupMap.put("paper", paperBackupList);

        // todo 备用题卡

        //3.5
        String variableContent = examPrintPlan.getVariableContent();
        String ordinaryContent = examPrintPlan.getOrdinaryContent();
        List<Map> variableList = JSONObject.parseArray(variableContent, Map.class);
        List<Map> otherList = new ArrayList<>();
        for (Map variable : variableList) {
            Map vMap = new HashMap();
            vMap.put("type", variable.get("type"));
            Map<String, String> urlMap = commonService.filePreview(variable.get("attachmentId").toString(), false);
            vMap.put("htmlUrl", urlMap.get("htmlUrl"));
            vMap.put("pdfUrl", urlMap.get("pdfUrl"));
            otherList.add(vMap);
        }
        List<Map> ordinaryList = JSONObject.parseArray(ordinaryContent, Map.class);
        for (Map ordinary : ordinaryList) {
            Map vMap = new HashMap();
            vMap.put("type", ordinary.get("type"));
            Map<String, String> urlMap = commonService.filePreview(ordinary.get("attachmentId").toString(), false);
            vMap.put("htmlUrl", urlMap.get("htmlUrl"));
            vMap.put("pdfUrl", urlMap.get("pdfUrl"));
            otherList.add(vMap);
        }
        finalMap.put("other", otherList);
        return finalMap;
    }

    /**
     * 根据考场生成试卷map
     *
     * @param schoolId
     * @param examDetailId
     * @return
     */
    public Map<String, String> mapCourseUrl(Long schoolId, Long examDetailId) {
        List<Map<String, String>> taskDetails = examTaskDetailService.listByExamDetailId(schoolId, examDetailId);
        Map<String, String> map = new HashMap<>();
        for (Map<String, String> taskDetail : taskDetails) {
            String courseCode = taskDetail.get("courseCode");
            String paperNumber = taskDetail.get("paperNumber");
            String paperAttachmentIds = taskDetail.get("paperAttachmentIds");
            List<Map> attachementIds = JSONObject.parseArray(paperAttachmentIds, Map.class);
            for (Map attaMap : attachementIds) {
                String name = attaMap.get("name").toString();
                String attachmentId = attaMap.get("attachmentId").toString();
                BasicAttachment attachment = basicAttachmentService.getById(attachmentId);
                String url = commonService.filePreview(attachment.getPath());
                StringJoiner sj = new StringJoiner(":");
                String key = sj.add(courseCode).add(paperNumber).add(name).toString();
                map.put(key, url);
            }
        }
        return map;
    }
}
