package com.qmth.distributed.print.business.templete.service.impl;

import cn.hutool.core.util.ZipUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.oss.common.utils.BinaryUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.dto.ExaminationExportDto;
import com.qmth.distributed.print.business.bean.dto.FieldsDto;
import com.qmth.distributed.print.business.bean.dto.PdfDto;
import com.qmth.distributed.print.business.bean.params.ArraysParams;
import com.qmth.distributed.print.business.bean.params.SerialNumberParams;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.*;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.service.TaskLogicService;
import com.qmth.distributed.print.business.util.ConvertUtil;
import com.qmth.distributed.print.business.util.CreatePdfUtil;
import com.qmth.distributed.print.business.util.ExcelUtil;
import com.qmth.distributed.print.business.util.OssUtil;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import com.qmth.distributed.print.common.util.FileUtil;
import com.qmth.distributed.print.common.util.HexUtils;
import com.qmth.distributed.print.common.util.JacksonUtil;
import com.qmth.distributed.print.common.util.ResultUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.support.atomic.RedisAtomicLong;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.FileCopyUtils;

import javax.annotation.Resource;
import java.io.*;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description: 任务处理逻辑impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
@Service
public class TaskLogicServiceImpl implements TaskLogicService {
    private final static Logger log = LoggerFactory.getLogger(TaskLogicServiceImpl.class);

    @Resource
    CommonService commonService;

    @Resource
    ExamDetailService detailService;

    @Resource
    ExamDetailCourseService detailCourseService;

    @Resource
    ExamPrintPlanService examPrintPlanService;

    @Resource
    ExamTaskService examTaskService;

    @Resource
    ExamTaskDetailService examTaskDetailService;

    @Resource
    ExamCardService examCardService;

    @Resource
    ExamCardDetailService examCardDetailService;

    @Resource
    ExamStudentService examStudentService;

    @Resource
    BasicAttachmentService basicAttachmentService;

    @Resource
    ExamDetailService examDetailService;

    @Resource
    OssUtil ossUtil;

    @Resource
    CacheService cacheService;

    @Resource
    BasicCourseService basicCourseService;

    @Resource
    CreatePdfUtil createPdfUtil;

    @Resource
    BasicTemplateService basicTemplateService;

    @Resource
    RedisTemplate<String, Object> redisTemplate;


    /**
     * 创建pdf逻辑
     *
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> executeCreatePdfLogic(Map<String, Object> map) throws IOException {
        List<BasicAttachment> basicAttachmentList = null;
        try {
            TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
            SysUser sysUser = (SysUser) map.get(SystemConstant.USER);
            Long examDetailId = Objects.nonNull(map.get("examDetailId")) ? (Long) map.get("examDetailId") : null;
            Long schoolId = (Long) map.get("schoolId");
            List<Long> examDetailCourseIds = Objects.nonNull(map.get("examDetailCourseIds")) ? (List<Long>) map.get("examDetailCourseIds") : null;
            String paperTypeParam = Objects.nonNull(map.get("paperType")) ? (String) map.get("paperType") : null;

            //这里为保存附件html的逻辑
            //查询题卡详情是否有附件id，有的话则把以前的附件删除
            Set<Long> attachmentIds = null;
            Set<File> ftlList = null;

            //查询printPlan
            ExamPrintPlan examPrintPlan = examPrintPlanService.getById(tbTask.getPrintPlanId());
            if (Objects.isNull(examPrintPlan)) {
                throw ExceptionResultEnum.ERROR.exception("印刷计划为空");
            }

            BasicSchool basicSchool = cacheService.schoolCache(examPrintPlan.getSchoolId());

            //查询examDetail
            QueryWrapper<ExamDetail> examDetailQueryWrapper = new QueryWrapper<>();
            examDetailQueryWrapper.lambda().eq(ExamDetail::getSchoolId, sysUser.getSchoolId())
                    .eq(ExamDetail::getPrintPlanId, tbTask.getPrintPlanId());
            if (Objects.nonNull(examDetailId)) {
                examDetailQueryWrapper.lambda().eq(ExamDetail::getId, examDetailId);
            }
            List<ExamDetail> examDetailList = detailService.list(examDetailQueryWrapper);
            if (Objects.isNull(examDetailList) || examDetailList.size() == 0) {
                throw ExceptionResultEnum.ERROR.exception("考务计划为空");
            }
            attachmentIds = Objects.isNull(attachmentIds) ? attachmentIds = new HashSet<>() : attachmentIds;
            ftlList = Objects.isNull(ftlList) ? ftlList = new HashSet<>() : ftlList;
            for (ExamDetail examDetail : examDetailList) {
                //查询examDetailCourse
                QueryWrapper<ExamDetailCourse> examDetailCourseQueryWrapper = new QueryWrapper<>();
                if (Objects.nonNull(examDetailCourseIds) && examDetailCourseIds.size() > 0) {
                    examDetailCourseQueryWrapper.lambda().in(ExamDetailCourse::getId, examDetailCourseIds);
                } else {
                    examDetailCourseQueryWrapper.lambda().eq(ExamDetailCourse::getExamDetailId, examDetail.getId());
                }
                List<ExamDetailCourse> examDetailCourseList = detailCourseService.list(examDetailCourseQueryWrapper);
                if (Objects.nonNull(examDetailCourseList) && examDetailCourseList.size() > 0) {
                    List<PdfDto> variablePdfList = new ArrayList<>();//变量印品(签到表、卷袋贴)
                    List<PdfDto> ordinaryPdfList = new ArrayList<>();//普通印品(登记表)
                    List<PdfDto> paperPdfList = new ArrayList<>();//所有试卷
                    List<PdfDto> examStudentPdfList = new ArrayList<>();//所有题卡
                    List<PdfDto> backupPaperPdfList = new ArrayList<>();//备份试卷
                    List<PdfDto> cardPdfList = new ArrayList<>();//备份题卡

                    if (Objects.nonNull(examPrintPlan.getOrdinaryContent())) {
                        //获取普通印品
                        JSONArray jsonArrayOrdinary = JSONArray.parseArray(examPrintPlan.getOrdinaryContent());
                        for (int i = 0; i < jsonArrayOrdinary.size(); i++) {
                            JSONObject jsonObjectOrdinary = jsonArrayOrdinary.getJSONObject(i);
                            Long attachmentId = Long.parseLong((String) jsonObjectOrdinary.get("attachmentId"));
                            BasicAttachment basicAttachment = basicAttachmentService.getById(attachmentId);
                            createPdfUtil.createCheckIn(basicAttachment, ordinaryPdfList, (Integer) jsonObjectOrdinary.get("backupCount"));
                        }
                    }

                    for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                        List<PdfDto> paperPdfTempList = new ArrayList<>();//所有试卷
                        List<PdfDto> examStudentTempPdfList = new ArrayList<>();//所有题卡
                        List<PdfDto> backupPaperTempPdfList = new ArrayList<>();//备份试卷
                        List<PdfDto> cardPdfTempList = new ArrayList<>();//备份题卡

                        //查询试卷
                        QueryWrapper<ExamTask> examTaskQueryWrapper = new QueryWrapper<>();
                        examTaskQueryWrapper.lambda().eq(ExamTask::getSchoolId, sysUser.getSchoolId())
                                .eq(ExamTask::getCourseCode, examDetailCourse.getCourseCode())
                                .eq(ExamTask::getCourseName, examDetailCourse.getCourseName())
                                .eq(ExamTask::getPaperNumber, examDetailCourse.getPaperNumber())
                                .eq(ExamTask::getEnable, true)
                                .eq(ExamTask::getStatus, ExamStatusEnum.FINISH);
                        List<ExamTask> examTaskList = examTaskService.list(examTaskQueryWrapper);
                        if (Objects.isNull(examTaskList) || examTaskList.size() == 0) {
                            throw ExceptionResultEnum.ERROR.exception("命题任务为空");
                        }
                        Set<Long> examTaskIds = examTaskList.stream().map(s -> s.getId()).collect(Collectors.toSet());

                        QueryWrapper<ExamTaskDetail> examTaskDetailQueryWrapper = new QueryWrapper<>();
                        examTaskDetailQueryWrapper.lambda().in(ExamTaskDetail::getExamTaskId, examTaskIds)
                                .eq(ExamTaskDetail::getEnable, true);
                        List<ExamTaskDetail> examTaskDetailList = examTaskDetailService.list(examTaskDetailQueryWrapper);
                        JSONObject jsonObject = new JSONObject();
                        JSONArray jsonArray = new JSONArray();
                        for (ExamTaskDetail examTaskDetail : examTaskDetailList) {
                            //查询题卡
                            ExamCard examCard = examCardService.getById(examTaskDetail.getCardId());
                            if (Objects.isNull(examCard)) {
                                throw ExceptionResultEnum.ERROR.exception("题卡为空");
                            }
                            QueryWrapper<ExamCardDetail> examCardDetailQueryWrapper = new QueryWrapper<>();
                            examCardDetailQueryWrapper.lambda().eq(ExamCardDetail::getCardId, examCard.getId());
                            List<ExamCardDetail> examCardDetailList = examCardDetailService.list(examCardDetailQueryWrapper);

                            //抽取卷型
                            String paperType = Objects.nonNull(paperTypeParam) ? paperTypeParam : createPdfUtil.getPaperType(examPrintPlan, examTaskDetail);
                            examTaskDetail.setRelatePaperType(paperType);

                            //获取试卷pdf
                            boolean tag = createPdfUtil.getPaperPdf(paperType, examTaskDetail, examPrintPlan.getBackupCount(), paperPdfTempList, backupPaperTempPdfList);
                            paperPdfList.addAll(paperPdfTempList);
                            backupPaperPdfList.addAll(backupPaperTempPdfList);

                            basicAttachmentList = Objects.isNull(basicAttachmentList) ? basicAttachmentList = new ArrayList<>() : basicAttachmentList;
                            for (ExamCardDetail examCardDetail : examCardDetailList) {
                                createPdfUtil.getCardAttachmentId(examCardDetail, attachmentIds);

                                //把模板页面上的 ${} 替换成实际内容
                                String cardContent = createPdfUtil.replaceHtmlTemplete(examCardDetail);
                                String studentContent = cardContent;

                                for (int i = 1; i <= examPrintPlan.getBackupCount(); i++) {
                                    basicAttachmentList.add(createPdfUtil.cardHtml(cardContent, paperType, examDetailCourse, examCard, jsonArray, sysUser.getId(), cardPdfTempList));
                                }
                                cardPdfList.addAll(cardPdfTempList);

                                //查询考生
                                QueryWrapper<ExamStudent> examStudentQueryWrapper = new QueryWrapper<>();
                                examStudentQueryWrapper.lambda().eq(ExamStudent::getSchoolId, sysUser.getSchoolId())
                                        .eq(ExamStudent::getExamDetailCourseId, examDetailCourse.getId());
                                List<ExamStudent> examStudentList = examStudentService.list(examStudentQueryWrapper);

                                if (Objects.nonNull(examPrintPlan.getVariableContent())) {
                                    //获取变量印品
                                    JSONArray jsonArrayVariable = JSONArray.parseArray(examPrintPlan.getVariableContent());
                                    for (int i = 0; i < jsonArrayVariable.size(); i++) {
                                        JSONObject jsonObjectVariable = jsonArrayVariable.getJSONObject(i);
                                        String type = (String) jsonObjectVariable.get("type");
                                        Long attachmentId = Long.parseLong((String) jsonObjectVariable.get("attachmentId"));
                                        BasicAttachment basicAttachment = basicAttachmentService.getById(attachmentId);
                                        ftlList.add(commonService.getFile(basicAttachment.getPath(), false));
                                        if (Objects.nonNull(type) && Objects.equals(type.toUpperCase(), "SIGN")) {//签到表
                                            createPdfUtil.createSignBook(basicAttachment, basicSchool.getName(), examDetail, examDetailCourse, examStudentList, variablePdfList, (Integer) jsonObjectVariable.get("backupCount"));
                                        } else if (Objects.nonNull(type) && Objects.equals(type.toUpperCase(), "PACKAGE")) {//卷袋贴
                                            createPdfUtil.createPaperPackage(tag, basicAttachment, basicSchool.getName(), examDetail, examDetailCourse, examStudentList, variablePdfList, (Integer) jsonObjectVariable.get("backupCount"));
                                        }
                                    }
                                }

                                Collections.sort(variablePdfList, new Comparator<PdfDto>() {
                                    @Override
                                    public int compare(PdfDto o1, PdfDto o2) {
                                        return o1.getSequence() > o2.getSequence() ? 1 : -1;
                                    }
                                });

                                for (ExamStudent t : examStudentList) {
                                    basicAttachmentList.add(createPdfUtil.examStudentHtml(attachmentIds, studentContent, t, paperType, examCard, sysUser.getId(), examStudentTempPdfList));
                                }
                                examStudentPdfList.addAll(examStudentTempPdfList);
                                examStudentService.saveOrUpdateBatch(examStudentList);
                                jsonObject.put("card", jsonArray);
                                examCardDetail.setAttachmentId(jsonObject.toJSONString());
                            }
                            examCardDetailService.saveOrUpdateBatch(examCardDetailList);
                        }
                        examTaskDetailService.saveOrUpdateBatch(examTaskDetailList);

                        Integer paperPageA3 = 0;
                        Integer cardPageA3 = 0;
                        paperPageA3 = paperPageA3 + paperPdfTempList.stream().mapToInt(PdfDto::getPageCount).sum() + backupPaperTempPdfList.stream().mapToInt(PdfDto::getPageCount).sum();
                        cardPageA3 = cardPageA3 + examStudentTempPdfList.stream().mapToInt(PdfDto::getPageCount).sum() + cardPdfTempList.stream().mapToInt(PdfDto::getPageCount).sum();
                        examDetailCourse.setPaperPagesA3(paperPageA3);
                        examDetailCourse.setCardPagesA3(cardPageA3);
                    }
                    detailCourseService.saveOrUpdateBatch(examDetailCourseList);
                    //合并pdf
                    basicAttachmentList.add(createPdfUtil.mergePdf(tbTask, examDetail, sysUser.getId(), schoolId, variablePdfList, ordinaryPdfList, paperPdfList, examStudentPdfList, backupPaperPdfList, cardPdfList));
                }
            }
            map.computeIfAbsent("size", v -> examDetailList.size());
            //最后一步删除附件
            createPdfUtil.deleteAttachment(attachmentIds, ftlList);
        } catch (Exception e) {
            log.error("请求出错", e);
            basicAttachmentService.batchDeleteAttachment(basicAttachmentList);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Map<String, Object> executeExaminationLogic(Map<String, Object> map) throws Exception {
        List<ExaminationExportDto> examinationExportDtoList = examDetailService.findExaminationExportDtoDatasource(map);

        System.out.println("examinationResultList = " + JSON.toJSONString(examinationExportDtoList));

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        ExcelUtil.excelMake(ExaminationExportDto.class, examinationExportDtoList, outputStream);
        InputStream in = new ByteArrayInputStream(outputStream.toByteArray());

//        UploadFileEnum.FILE/
        LocalDateTime nowTime = LocalDateTime.now();
        StringJoiner stringJoiner = new StringJoiner("");
        stringJoiner.add(UploadFileEnum.FILE.getTitle()).add(File.separator);

        stringJoiner.add(String.valueOf(nowTime.getYear())).add(File.separator)
                .add(String.format("%02d", nowTime.getMonthValue())).add(File.separator)
                .add(String.format("%02d", nowTime.getDayOfMonth()));

        JSONObject jsonObject = new JSONObject();
        stringJoiner.add(File.separator).add(SystemConstant.getUuid()).add(".").add(SystemConstant.XLSX);

        String dirName = stringJoiner.toString().replaceAll("\\\\", "/");

        ossUtil.ossUpload(dirName, in, null);
        jsonObject.put(SystemConstant.TYPE, SystemConstant.OSS);
        jsonObject.put(SystemConstant.PATH, dirName);
        jsonObject.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.FILE);
        map.put("path", jsonObject.toString());
        return map;
    }

    @Transactional
    @Override
    public Map<String, Object> executeImportExaminationLogic(Map<String, Object> map) throws Exception {
        TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
        InputStream inputStream = (InputStream) map.get("inputStream");
        Long printPlanId = tbTask.getPrintPlanId();
        Long userId = tbTask.getCreateId();
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(printPlanId);

        if (examPrintPlan == null) {
            throw ExceptionResultEnum.ERROR.exception("印刷计划不存在printPlanId ： " + printPlanId);
        }

        PrintPlanStatusEnum printPlanStatus = examPrintPlan.getStatus();
        if (PrintPlanStatusEnum.NEW != printPlanStatus && PrintPlanStatusEnum.READY != printPlanStatus) {
            throw ExceptionResultEnum.ERROR.exception(printPlanStatus + "状态下的印刷计划不允许导入");
        }

        String printPlanName = examPrintPlan.getName();
        Long schoolId = tbTask.getSchoolId();
        // 该学校有效考务数据
        List<FieldsDto> fieldsDtoList = examDetailService.findExaminationFields(schoolId);
        if (fieldsDtoList.size() == 0) {
            throw ExceptionResultEnum.ERROR.exception("该学校没有设置考务数据");
        }
        String importFilePath = tbTask.getImportFilePath();
        Map importFilePathMap = JSONObject.parseObject(importFilePath);
        String path = String.valueOf(importFilePathMap.get("path"));
        System.out.println("path = " + path);
        Workbook workbook;
        if (path.endsWith(SystemConstant.XLSX)) {
            workbook = new XSSFWorkbook(inputStream);
        } else if (path.endsWith("xls")) {
            workbook = new HSSFWorkbook(inputStream);
        } else {
            throw ExceptionResultEnum.ERROR.exception("文件格式异常");
        }
        // 读取第一个工作表
        Sheet sheet = workbook.getSheetAt(0);
        // 获取sheet行数
        int totalRows = sheet.getPhysicalNumberOfRows();
        // 获取sheet列数
        int totalCells = 0;
        if (totalRows > 1 && sheet.getRow(0) != null) {
            totalCells = sheet.getRow(0).getPhysicalNumberOfCells();
        }
        System.out.println("行 ： " + totalRows + "\n列 ： " + totalCells);
        Row head = sheet.getRow(0);
        List<String> headList = new ArrayList<>();
        // 将必填字段匹配excel解析的表头索引
        for (int i = 0; i < totalCells; i++) {
            String cellValue = head.getCell(i).getStringCellValue();
            for (FieldsDto fieldsDto : fieldsDtoList) {
                if (cellValue.equals(fieldsDto.getName())) {
                    fieldsDto.setIndex(i);
                }
            }
            headList.add(cellValue);
        }

        // 搜索所有有效字段 excel中的表头是否包含
        for (FieldsDto fieldsDto : fieldsDtoList) {
            if (!headList.contains(fieldsDto.getName())) {
                System.out.println("exception " + fieldsDto.getName());
                throw ExceptionResultEnum.ERROR.exception("学校考务必填字段 ：'" + fieldsDto.getName() + "' 不存在");
            }
        }

        System.out.println("headList : " + headList);
        System.out.println("fieldsDtoList : " + JSONObject.toJSONString(fieldsDtoList));

        List<Map<String, Object>> dataList = new ArrayList<>();

        for (int r = 1; r < totalRows; r++) {
            Row row = sheet.getRow(r);

            String studentCode = null;
            String studentName = null;
            String courseCode = null;
            String courseName = null;
            String examPlace = null;
            String examRoom = null;
            String examDate = null;
            String examTime = null;
            String paperNumber = null;
            List<FieldsDto> secondaryFieldList = new ArrayList<>(); // 备选字段
            for (FieldsDto fieldsDto : fieldsDtoList) {
                String name = fieldsDto.getName();
                String code = fieldsDto.getCode();
                int index = fieldsDto.getIndex();
                String level = fieldsDto.getLevel();

                Cell cell = row.getCell(index);
                String cellValue = String.valueOf(ExcelUtil.convert(cell));
                // TODO: 2021/4/20 可以优化 
                if (ExaminationDBFieldsEnum.STUDENT_CODE.getDesc().equals(name)) {
                    studentCode = cellValue;
                    ConvertUtil.verifyLength(studentCode, 10, ExaminationDBFieldsEnum.STUDENT_CODE.getDesc());
                } else if (ExaminationDBFieldsEnum.STUDENT_NAME.getDesc().equals(name)) {
                    studentName = cellValue;
                    ConvertUtil.verifyLength(studentName, 15, ExaminationDBFieldsEnum.STUDENT_NAME.getDesc());
                } else if (ExaminationDBFieldsEnum.COURSE_CODE.getDesc().equals(name)) {
                    courseCode = cellValue;
                    ConvertUtil.verifyLength(courseCode, 10, ExaminationDBFieldsEnum.COURSE_CODE.getDesc());
                } else if (ExaminationDBFieldsEnum.COURSE_NAME.getDesc().equals(name)) {
                    courseName = cellValue;
                    ConvertUtil.verifyLength(courseName, 20, ExaminationDBFieldsEnum.COURSE_NAME.getDesc());
                } else if (ExaminationDBFieldsEnum.EXAM_PLACE.getDesc().equals(name)) {
                    examPlace = cellValue;
                    ConvertUtil.verifyLength(examPlace, 10, ExaminationDBFieldsEnum.EXAM_PLACE.getDesc());
                } else if (ExaminationDBFieldsEnum.EXAM_ROOM.getDesc().equals(name)) {
                    examRoom = cellValue;
                    ConvertUtil.verifyLength(examRoom, 10, ExaminationDBFieldsEnum.EXAM_ROOM.getDesc());
                } else if (ExaminationDBFieldsEnum.EXAM_DATE.getDesc().equals(name)) {
                    examDate = cellValue;
                } else if (ExaminationDBFieldsEnum.EXAM_TIME.getDesc().equals(name)) {
                    examTime = cellValue;
                } else if (ExaminationDBFieldsEnum.PAPER_NUMBER.getDesc().equals(name)) {
                    paperNumber = cellValue;
                    ConvertUtil.verifyLength(paperNumber, 15, ExaminationDBFieldsEnum.PAPER_NUMBER.getDesc());
                } else {
                    if ("primary".equals(level)) {
                        throw ExceptionResultEnum.ERROR.exception("有数据库不需要的必选字段 ： " + name);
                    }
                    if ("secondary".equals(level)) {
                        // 备选字段处理
                        FieldsDto secondaryField = new FieldsDto();
                        secondaryField.setLevel(level);
                        secondaryField.setCode(code);
                        secondaryField.setName(name);
                        secondaryField.setEnable(true);
                        secondaryField.setValue(cellValue);
                        secondaryFieldList.add(secondaryField);
                    }
                }
            }
            // 校验试卷编号
            examTaskService.verifyCourseCodeByPaperNumber(schoolId, paperNumber, courseCode);

            // 校验课程信息
            basicCourseService.verifyCourseInfo(schoolId, courseCode, courseName, userId);

            // 解析时间
            Map<String, Object> timeMap = ConvertUtil.analyzeStartAndEndTime(examDate, examTime);
            String examStartTime = String.valueOf(timeMap.get("startTime"));
            String examEndTime = String.valueOf(timeMap.get("endTime"));

            Map<String, Object> dataMap = new HashMap<>();
            dataMap.put("studentCode", studentCode);
            dataMap.put("studentName", studentName);
            dataMap.put("courseCode", courseCode);
            dataMap.put("courseName", courseName);
            dataMap.put("examPlace", examPlace);
            dataMap.put("examRoom", examRoom);
            dataMap.put("examStartTime", examStartTime);
            dataMap.put("examEndTime", examEndTime);
            dataMap.put("paperNumber", paperNumber);
            dataMap.put("secondaryFieldList", secondaryFieldList);
            dataMap.put("schoolId", schoolId);
            dataMap.put("printPlanId", printPlanId);
            dataMap.put("printPlanName", printPlanName);
            dataList.add(dataMap);
        }
        System.out.println(JSON.toJSONString(dataList));
        // 校验课程代码和试卷编号在印刷计划下是1对1的关系
        List<String> courseCodeList = dataList.stream().map(e -> String.valueOf(e.get("courseCode"))).distinct().collect(Collectors.toList());
        List<String> paperNumberList = dataList.stream().map(e -> String.valueOf(e.get("paperNumber"))).distinct().collect(Collectors.toList());
        for (String courseCode : courseCodeList) {
            List<String> tmp = dataList.stream().filter(e -> courseCode.equals(String.valueOf(e.get("courseCode"))))
                    .map(e -> String.valueOf(e.get("paperNumber"))).distinct().collect(Collectors.toList());
            if (tmp.size() != 1) {
                throw ExceptionResultEnum.ERROR.exception("课程代码为 " + courseCode + ",对应多个试卷编号 : " + tmp);
            }
        }
        for (String paperNumber : paperNumberList) {
            List<String> tmp = dataList.stream().filter(e -> paperNumber.equals(String.valueOf(e.get("paperNumber"))))
                    .map(e -> String.valueOf(e.get("courseCode"))).distinct().collect(Collectors.toList());
            if (tmp.size() != 1) {
                throw ExceptionResultEnum.ERROR.exception("试卷编号为 " + paperNumber + ",对应多个课程代码 : " + tmp);
            }
        }

        // 删除印刷计划下的考务数据
        examDetailService.deleteExaminationData(printPlanId);


        SerialNumberParams serialNumberParams = new SerialNumberParams("packageCode-", "p", 6);
        String key = serialNumberParams.getModel() + serialNumberParams.getPrefix();
        RedisAtomicLong counter = new RedisAtomicLong(key, Objects.requireNonNull(redisTemplate.getConnectionFactory()));
        Long value = counter.get();
        try {
            // 组装exam_detail数据
            List<Long> examDetailIdList = examDetailService.disposeExamDetailByExaminationExcel(dataList, userId, serialNumberParams);
            // 组装exam_detail_course数据
            examDetailService.disposeExamDetailCourseByExaminationExcel(dataList, userId);
            // 组装exam_student数据
            examDetailService.disposeExamStudentByExaminationExcel(dataList, userId);

            // 按照考场检验命题任务是否全部完成，完成生成pdf
            SysUser user = (SysUser) map.get(SystemConstant.USER);
            for (Long examDetailId : examDetailIdList) {
                examTaskService.checkDataByExamination(examDetailId, user);
            }

//            List<Map<String, Object>> checkList = dataList.stream().flatMap(e -> {
//                Map<String, Object> tmp = new HashMap<>();
//                tmp.put("schoolId", e.get("schoolId"));
//                tmp.put("courseCode", e.get("courseCode"));
//                tmp.put("paperNumber", e.get("paperNumber"));
//                return Stream.of(tmp);
//            }).distinct().collect(Collectors.toList());
//            for (Map<String, Object> stringObjectMap : checkList) {
//                Long checkSchoolId = SystemConstant.convertIdToLong(String.valueOf(stringObjectMap.get("schoolId")));
//                String checkCourseCode = String.valueOf(stringObjectMap.get("courseCode"));
//                String checkPaperNumber = String.valueOf(stringObjectMap.get("paperNumber"));
//                SysUser user = (SysUser) map.get(SystemConstant.USER);
//                user.setSchoolId(schoolId);
//                commonService.checkData(checkSchoolId, checkCourseCode, checkPaperNumber, user);
//            }
            map.put("dataCount", dataList.size());
        } catch (Exception e) {
            redisTemplate.opsForValue().set(key, value);
            throw new RuntimeException(e);
        }

        return map;
    }

    /**
     * 下载pdf
     *
     * @param map
     * @return
     * @throws IOException
     */
    @Override
    @Transactional
    public Map<String, Object> executeDownloadPdfLogic(Map<String, Object> map) throws Exception {
        try {
            TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
            JSONArray jsonArray = JSONArray.parseArray(tbTask.getRemark());
            ArraysParams arraysParams = new ArraysParams(jsonArray.toArray(new Long[jsonArray.size()]));
            if (Objects.isNull(arraysParams) || arraysParams.getIds().length == 0) {
                throw ExceptionResultEnum.ERROR.exception("数组id为空");
            }

            //查询examDetail
            List<ExamDetail> examDetailList = examDetailService.listByIds(Arrays.asList(arraysParams.getIds()));
            if (Objects.nonNull(examDetailList)) {
                Set<Long> attachmentIds = examDetailList.stream().map(ExamDetail::getAttachmentId).collect(Collectors.toSet());
                List<BasicAttachment> basicAttachmentList = basicAttachmentService.listByIds(attachmentIds);

                if (Objects.nonNull(basicAttachmentList) && basicAttachmentList.size() > 0) {
                    LocalDateTime nowTime = LocalDateTime.now();
                    StringJoiner stringJoiner = new StringJoiner("")
                            .add(SystemConstant.TEMP_FILES_DIR).add(File.separator);
                    StringJoiner dirName = new StringJoiner("")
                            .add(UploadFileEnum.FILE.getTitle()).add(File.separator)
                            .add(String.valueOf(nowTime.getYear())).add(File.separator)
                            .add(String.format("%02d", nowTime.getMonthValue())).add(File.separator)
                            .add(String.format("%02d", nowTime.getDayOfMonth()))
                            .add(File.separator).add(SystemConstant.getUuid()).add(SystemConstant.ZIP_PREFIX);
                    File zipFile = new File(stringJoiner.toString() + dirName.toString());
                    if (!zipFile.getParentFile().exists()) {
                        zipFile.getParentFile().mkdirs();
                        zipFile.createNewFile();
                    }
                    List<File> sourceFiles = new ArrayList<>();
                    for (BasicAttachment basicAttachment : basicAttachmentList) {
                        JSONObject jsonObject = JSONObject.parseObject(basicAttachment.getPath());
                        StringJoiner stringJoinerPdf = new StringJoiner("")
                                .add(SystemConstant.TEMP_FILES_DIR).add(File.separator)
                                .add((String) jsonObject.get("path"));
                        sourceFiles.add(ossUtil.ossDownload((String) jsonObject.get("path"), stringJoinerPdf.toString()));
                    }
                    FileUtil.doZip(zipFile, sourceFiles);
                    ossUtil.ossUpload(dirName.toString(), zipFile, BinaryUtil.toBase64String(HexUtils.decodeHex(DigestUtils.md5Hex(new FileInputStream(zipFile)))));
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(SystemConstant.PATH, dirName.toString());
                    jsonObject.put(SystemConstant.TYPE, SystemConstant.OSS);
                    jsonObject.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.FILE);
                    tbTask.setResultFilePath(jsonObject.toJSONString());
                    for (File file : sourceFiles) {
                        file.delete();
                    }
                    zipFile.delete();
                }
                map.computeIfAbsent(SystemConstant.SIZE, v -> attachmentIds.size());
            }
        } catch (Exception e) {
            log.error("请求出错", e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    @Override
    @Transactional
    public Map<String, Object> executeExportSampleLogic(Map<String, Object> map) throws IOException {
        int count = 0;
        TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
        LocalDateTime nowTime = LocalDateTime.now();
        StringJoiner zipJoiner = new StringJoiner("")
                .add(SystemConstant.TEMP_FILES_DIR).add(File.separator);
        StringJoiner dirName = new StringJoiner("")
                .add(UploadFileEnum.FILE.getTitle()).add(File.separator)
                .add(String.valueOf(nowTime.getYear())).add(File.separator)
                .add(String.format("%02d", nowTime.getMonthValue())).add(File.separator)
                .add(String.format("%02d", nowTime.getDayOfMonth()))
                .add(File.separator).add(SystemConstant.getUuid()).add(SystemConstant.ZIP_PREFIX);
        String dirNameTmp = dirName.toString().replaceAll("\\\\", "/");
        File zipFile = new File(zipJoiner.toString() + dirNameTmp);
        if (!zipFile.getParentFile().exists()) {
            zipFile.getParentFile().mkdirs();
            zipFile.createNewFile();
        }

        // 跟文件路径
        String zipLocalRootPath = SystemConstant.TEMP_FILES_DIR + File.separator + System.currentTimeMillis();
        Long[] ids = (Long[]) map.get("ids");
        for (Long id : ids) {
            ExamTask examTask = examTaskService.getById(id);
            if (Objects.isNull(examTask)) {
                throw ExceptionResultEnum.ERROR.exception("未找到命题任务");
            }
            String examTaskSign = examTask.getSchoolId() + "-" + examTask.getCourseName() + "-" + examTask.getPaperNumber();
            String firstPath = zipLocalRootPath + File.separator + examTaskSign;
            List<ExamTaskDetail> examTaskDetailList = examTaskDetailService.list(new QueryWrapper<ExamTaskDetail>().lambda().eq(ExamTaskDetail::getExamTaskId, id));
            if (examTaskDetailList.size() != 1) {
                throw ExceptionResultEnum.ERROR.exception("获取命题任务详情失败 命题任务id ： " + id);
            }
            ExamTaskDetail examTaskDetail = examTaskDetailList.get(0);


            // 处理试卷样品
            List<Map> paperInfo = JSONObject.parseArray(examTaskDetail.getPaperAttachmentIds(), Map.class);
            System.out.println("paperInfo = " + JSON.toJSONString(paperInfo));
            Set<Long> attPaperIds = new HashSet<>();
            for (Map paperMap : paperInfo) {
                if (Objects.isNull(paperMap.get("attachmentId"))) {
                    throw ExceptionResultEnum.ERROR.exception("未找到附件id");
                }
                Long attachmentId = Long.valueOf(String.valueOf(paperMap.get("attachmentId")));
                attPaperIds.add(attachmentId);
            }
            List<BasicAttachment> paperAttachmentList = basicAttachmentService.listByIds(attPaperIds);
            if (Objects.nonNull(paperAttachmentList)) {
                for (BasicAttachment paperAttachment : paperAttachmentList) {
                    JSONObject jsonObject = JSONObject.parseObject(paperAttachment.getPath());
                    String paperPath = firstPath + File.separator + "试卷" + File.separator + paperAttachment.getName() + paperAttachment.getType();
                    ossUtil.ossDownload((String) jsonObject.get("path"), paperPath);
                    count++;
                }
            }

            // 处理审核样品
            List<Map> confirmInfo = JSONObject.parseArray(examTaskDetail.getPaperConfirmAttachmentIds(), Map.class);
            System.out.println("confirmInfo = " + JSON.toJSONString(confirmInfo));
            Set<Long> attConfirmIds = new HashSet<>();
            for (Map confirmMap : confirmInfo) {
                if (Objects.isNull(confirmMap.get("attachmentId"))) {
                    throw ExceptionResultEnum.ERROR.exception("未找到附件id");
                }
                Long attachmentId = Long.valueOf(String.valueOf(confirmMap.get("attachmentId")));
                attConfirmIds.add(attachmentId);
            }
            List<BasicAttachment> confirmAttachmentList = basicAttachmentService.listByIds(attConfirmIds);
            if (Objects.nonNull(confirmAttachmentList)) {
                for (BasicAttachment confirmAttachment : confirmAttachmentList) {
                    JSONObject jsonObject = JSONObject.parseObject(confirmAttachment.getPath());
                    String confirmPath = firstPath + File.separator + "审核样本" + File.separator + confirmAttachment.getName() + confirmAttachment.getType();
                    ossUtil.ossDownload((String) jsonObject.get("path"), confirmPath);
                    count++;
                }
            }

            // 处理题卡
            Long cardId = examTaskDetail.getCardId();
            ExamCard examCard = examCardService.getById(cardId);
            if (Objects.isNull(examCard)) {
                throw ExceptionResultEnum.ERROR.exception("找不到答题卡 cardId = " + cardId);
            }
            MakeMethodEnum makeMethodEnum = examCard.getMakeMethod();
            if (MakeMethodEnum.SELECT.equals(makeMethodEnum)) {
                Long templateId = examCard.getTemplateId();
                if (templateId == null || templateId == 0) {
                    throw ExceptionResultEnum.ERROR.exception("找不到题卡对应的模板 templateId = " + templateId);
                }
                BasicTemplate basicTemplate = basicTemplateService.getById(templateId);
                if (Objects.isNull(basicTemplate)) {
                    throw ExceptionResultEnum.ERROR.exception("找不到模板信息 templateId = " + templateId);
                }
                Long attachmentId = basicTemplate.getAttachmentId();
                if (attachmentId == null || attachmentId == 0) {
                    throw ExceptionResultEnum.ERROR.exception("找不到模板对应的附件 templateId = " + templateId);
                }
                BasicAttachment cardAttachment = basicAttachmentService.getById(attachmentId);
                if (Objects.isNull(cardAttachment)) {
                    throw ExceptionResultEnum.ERROR.exception("找不到附件 attachmentId = " + attachmentId);
                }
                JSONObject jsonObject = JSONObject.parseObject(cardAttachment.getPath());
                String cardPath = firstPath + File.separator + "题卡" + File.separator + cardAttachment.getName() + cardAttachment.getType();
                ossUtil.ossDownload((String) jsonObject.get("path"), cardPath);
                count++;
            } else {
                List<ExamCardDetail> examCardDetailList = examCardDetailService.list(new QueryWrapper<ExamCardDetail>().lambda().eq(ExamCardDetail::getCardId, cardId));
                if (examCardDetailList.size() != 1) {
                    throw ExceptionResultEnum.ERROR.exception("题卡明细信息异常 card_id = " + cardId);
                }
                ExamCardDetail examCardDetail = examCardDetailService.getByCardId(cardId);
                String htmlContent = examCardDetail.getHtmlContent();
                byte[] bytes = htmlContent.getBytes();
                String cardPath = firstPath + File.separator + "题卡" + File.separator + examCard.getTitle() + SystemConstant.HTML_PREFIX;
                File localFile = new File(cardPath);
                if (!localFile.getParentFile().exists()) {
                    localFile.getParentFile().mkdirs();
                }
                FileCopyUtils.copy(bytes, localFile);
                count++;
            }
        }
        ZipUtil.zip(zipLocalRootPath, zipFile.getPath(), true);
        ossUtil.ossUpload(dirNameTmp, zipFile, BinaryUtil.toBase64String(HexUtils.decodeHex(DigestUtils.md5Hex(new FileInputStream(zipFile)))));
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(SystemConstant.PATH, dirNameTmp);
        jsonObject.put(SystemConstant.TYPE, SystemConstant.OSS);
        jsonObject.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.FILE);
        tbTask.setResultFilePath(jsonObject.toJSONString());
        ConvertUtil.delFolder(zipLocalRootPath);
        zipFile.delete();
        map.put("count", count);
        return map;
    }
}
